@@ -1,27 +1,20 @@
-from lxml import html
 import requests
 
 class AuthenticationError(Exception):
   pass
-
-class AutologinSession(requests.Session):
+class AutologinHandler:
   """
-  Maintains an authenticated session to a web system. This class intercepts any 
-  requests made in a requests.Session and ensures that we log in when 
-  redirected to the login page.
-
-  This is an abstract class.
+  An abstract class for a handler for the AutologinSession class.
   """
 
-  def __init__(self):
-    super().__init__()
-    self.__logging_in = False
-
-  def login(self, response, args=None, kwargs=None):
+  def login(self, session, response, args=None, kwargs=None):
     """
     Performs a login based on the response from a request.
-    args and kwargs are the options from the request triggering the login 
-    procedure, this is so that we can redo that request after loggin in.
+    - `session` is an instance of requests.Session, most likely an instance of 
+      AutologinSession.
+    - `response` is the response of the latest request.
+    - `args` and `kwargs` are the options from the latest request, this is so 
+      that we can redo that request after logging in.
 
     Raises an AuthenticationError exception if authentication fails.
     """
@@ -30,20 +23,32 @@ class AutologinSession(requests.Session):
   def need_login(self, response):
     """
     Checks a response to determine if logging in is needed,
-    returns True if needed
+    returns True if needed.
     """
     raise NotImplementedError()
 
+class AutologinSession(requests.Session):
+  """
+  Maintains an authenticated session to a web system. This class intercepts any 
+  requests made in a requests.Session and ensures that we log in when 
+  redirected to the login page.
+  """
+
+  def __init__(self, handlers):
+    """
+    Takes a list of handlers. A handler should derive from AutologinHandler.
+    """
+    super().__init__()
+    self.__handlers = handlers
+
   def request(self, *args, **kwargs):
     """
-    Wrapper around requests.Session.request(...) to check if logging in
-    is needed.
+    Wrapper around requests.Session.request(...) to check we must log in.
     """
     response = super().request(*args, **kwargs)
     
-    if not self.__logging_in and self.need_login(response):
-      self.__logging_in = True
-      response = self.login(response, args, kwargs)
-      self.__logging_in = False
+    for handler in self.__handlers:
+      if handler.need_login(response):
+        response = handler.login(self, response, args, kwargs)
 
     return response
